import * as React from "react";
import {Conditional, PaddedPaper} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {Card, Divider, Grid, makeStyles, Typography} from "@material-ui/core";
import {green, red} from "@material-ui/core/colors";
import {BlockPickingDataField} from "./BlockPickingDataField";
import {t} from "@lingui/macro";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";





type BlockInfoPanelProps = {
    model: BlockInfo | undefined,
    hideTrayNumber: boolean,
    hideJobNumber: boolean
};

const useStyles = makeStyles((theme) => ({
    divider: {
        marginBottom: theme.spacing(1),
    },
    jobInfoSection: {
        marginBottom: theme.spacing(1),
        marginTop: theme.spacing(1),
    },
    jobInfoCard: {
        width: "25%",
    },
    eyeInfoSection: {
        marginBottom: theme.spacing(1),
        marginTop: theme.spacing(1),
    },
    eyeInfoCard: {
        width: "45%",
        height: '20rem'
    },
    eyeTagCard: {
        width: "45%"
    },
    textFalse: {
        color: red[300]
    },
    textTrue: {
        color: green[300]
    },
    tagText: {
        textAlign: 'center',
    },
    card: {
        width: "100%",
        height: "100%"
    },
}));

export const BlockInfoPanel: React.FunctionComponent<BlockInfoPanelProps> = observer((props) => {

    const {model, hideTrayNumber, hideJobNumber} = props;
    const classes = useStyles();

    const getBlockPieceCode = React.useCallback((diameter:string|null, curvature:string|null) => {
        if (!diameter && !curvature) {
            return t`No Information Available`;
        }
        const diam_text = diameter ? diameter : t`N/A`;
        const curve_text = curvature ? curvature : t`N/A`;

        return `${curve_text}-${diam_text}`;

    }, []);

    if (!model) {
        return null;
    }

    const reworkText = model.rework ? t`side` + `: ${model.rework}` : "";
    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();
    const leftLensConfig = model.do_left ? { label: t`Left Eye`, side: 'left', diameter: model.diameter.left, curvature: model.curvature.left} : null;
    const rightLensConfig = model.do_right ? { label: t`Right Eye`, side: 'right', diameter: model.diameter.right, curvature: model.curvature.right} : null;
    const lensConfigArr = displayRightLensFirst ? [rightLensConfig, leftLensConfig] : [leftLensConfig, rightLensConfig];

    return (
        <PaddedPaper>
            <Grid container direction='column'>
                <Grid className={classes.jobInfoSection} container item justifyContent='space-evenly'>
                    <Conditional condition={!hideJobNumber}>
                        <Grid item className={classes.jobInfoCard}>
                            <BlockPickingDataField label={t`Job Number`} value={model.jobNumber}/>
                        </Grid>
                    </Conditional>
                    <Conditional condition={!hideTrayNumber}>
                        <Grid item className={classes.jobInfoCard}>
                            <BlockPickingDataField label={t`Tray Number`} value={model.trayNumber}/>
                        </Grid>
                    </Conditional>
                    <Conditional condition={model.rework != null}>
                        <Grid item className={classes.jobInfoCard}>
                            <BlockPickingDataField label={t`Rework`} value={reworkText}/>
                        </Grid>
                    </Conditional>
                </Grid>
                <Divider className={classes.divider}/>
                <Grid className={classes.eyeInfoSection} container item justifyContent='space-evenly'>
                    {lensConfigArr.map(lensConfig => {
                        return <Grid item className={classes.eyeInfoCard}>
                            {!lensConfig ?
                                <BlockPickingDataField
                                    label={``}
                                    value={t`No Eye`}
                                    labelSize='h4'
                                    valueSize={'h2'}
                                />
                                : <BlockPickingDataField
                                    label={lensConfig.label}
                                    value={getBlockPieceCode(lensConfig.diameter, lensConfig.curvature)}
                                    labelSize='h4'
                                    valueSize={'h3'}
                                />
                            }
                        </Grid>;
                    })}
                </Grid>
                <Divider className={classes.divider}/>
                <Grid className={classes.eyeInfoSection} container item justifyContent='space-evenly'>
                    {lensConfigArr.map(lensConfig => {
                        return <Grid item className={classes.eyeTagCard}>
                            {!lensConfig
                                ? null
                                : <Card className={classes.card}>
                                    {
                                        model.tagValues.map(data => {
                                            return <Typography variant={'h4'} className={classes.tagText}>
                                                {`${data.tag} :  ${data[lensConfig.side]}`}
                                            </Typography>
                                        })
                                    }
                                </Card>
                            }
                        </Grid>
                    })}
                </Grid>
            </Grid>
        </PaddedPaper>
    );
});

export type BlockInfo = {
    jobNumber: string;
    trayNumber: string;
    do_left: boolean;
    do_right: boolean;
    diameter: {
        left: null | string;
        right: null | string;
    };
    curvature: {
        left: null | string;
        right: null | string;
    };
    rework: string | null;
    tagValues: TagValue[];
};

type TagValue = {
    name: string;
    left: string;
    right: string;
    [name: string]: string;
};
