import * as React from 'react';
import {t} from "@lingui/macro";
import {
    JobStateInfo,
    MachineHistory,
    MachineInfo,
    Side,
    sideDisplayText
} from "../../pages/job_history/JobHistoryOperatorForm";
import ReactTable from "react-table";
import {JobStateTable} from "./JobStateTable";
import {SidesPaperDisplay} from "../generic/SidesPaperDisplay";
import {makeStyles} from "@material-ui/core";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {WorkstationGenericWrapper} from "@rx-suite/ui";

type MachineHistoryTableProps = {
    machineHistory: MachineHistory;
};

const useStyles = makeStyles((theme) => ({
    tables: {
        margin: '1rem 0'
    }
}));


export const MachineHistoryTable: React.FunctionComponent<MachineHistoryTableProps> = ({ machineHistory }) => {

    const classes = useStyles();

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const compareByLatestTimeDesc = (a: JobStateInfo, b: JobStateInfo) => {

        // if the time does not exist, move it to the end of the list
        if (!a.state.time) return 1;
        if (!b.state.time) return -1;

        // if the time exists, compare it to the other time
        if (a.state.time > b.state.time) return -1;
        if (b.state.time > a.state.time) return 1;

        return 0; // times are equal
    };

    const getLatestJobState = (machineInfo: MachineInfo) => {

        // get the latest time of ANY job state in a machine
        // regardless of it being a left or right job state
        const leftJobStates = machineInfo.states.left;
        const rightJobStates = machineInfo.states.right;
        const allJobStates = leftJobStates.concat(rightJobStates);
        const sortedJobStates = allJobStates.sort(compareByLatestTimeDesc);
        const latestJobState = sortedJobStates[0];

        return (!latestJobState ? null : latestJobState);
    };

    const compareByLatestTimeAsc = (a: JobStateInfo, b: JobStateInfo) => {

        // if the time does not exist, move it to the end of the list
        if (!a.state.time) return 1;
        if (!b.state.time) return -1;

        // if the time exists, compare it to the other time
        if (a.state.time > b.state.time) return 1;
        if (b.state.time > a.state.time) return -1;

        return 0; // times are equal
    };

    const compareMachineTimeAsc = (a: MachineInfo, b: MachineInfo) => {

        // compare machines based on their latest job state time
        const latestJobStateA = getLatestJobState(a);
        const latestJobStateB = getLatestJobState(b);

        // if the job state does not exist, move it to the end of the list
        if (!latestJobStateA) return 1;
        if (!latestJobStateB) return -1;

        return compareByLatestTimeAsc(latestJobStateA, latestJobStateB);
    };

    const columns = [{
        Header: t`Machine Name`,
        accessor: 'name'
    }, {
        Header: t`Serial Number`,
        accessor: 'sn'
    }, {
        Header: t`Description`,
        accessor: 'description'
    }, {
        id: 'lastUpdate',
        Header: t`Last Update`,
        accessor: (d: MachineInfo) => {
            // display the latest time of ANY job state from this machine
            const latestJobState = getLatestJobState(d);

            if (!latestJobState) {
                return null;
            }
            const time = latestJobState.state.time;

            if (!time) {
                return null;
            }

            return systemSettingModel.printDateTime(new Date(time));
        }
    }];

    const machineHistoryValues = Object.values(machineHistory).slice();

    // sort the machines in ascending order based on its latest job state for display
    machineHistoryValues.sort(compareMachineTimeAsc);

    return <ReactTable
        data={machineHistoryValues}
        columns={columns}
        showPagination={true}
        minRows={machineHistoryValues.length <= 0 ? 10 : undefined} // if empty, pad the table with 10 rows
        pageSize={machineHistoryValues.length}
        showPageJump={false}
        showPageSizeOptions={false}
        showPaginationBottom={false}
        sortable={false}
        SubComponent={row => {

            return <WorkstationGenericWrapper><div className={classes.tables}>
                <SidesPaperDisplay
                    leftDisplay={{
                        label: sideDisplayText(Side.LEFT, row.original.isBoxCoater),
                        component: <JobStateTable
                             machineName={row.original.name}
                             side={Side.LEFT}
                             jobStates={row.original.states.left}
                             isBoxCoater={row.original.isBoxCoater}
                        />
                    }}
                    rightDisplay={{
                        label: sideDisplayText(Side.RIGHT, row.original.isBoxCoater),
                        component: <JobStateTable
                            machineName={row.original.name}
                            side={Side.RIGHT}
                            jobStates={row.original.states.right}
                            isBoxCoater={row.original.isBoxCoater}
                        />
                    }}
                    invertSides={displayRightLensFirst}
                />
            </div></WorkstationGenericWrapper>;
        }}
    />;
};
