import * as React from "react";
import {observer} from "mobx-react-lite";
import {Button, CircularProgress, Dialog, DialogContent, DialogTitle, IconButton, Typography} from "@material-ui/core";
import CloseIcon from "@material-ui/icons/Close";
import {ButtonField, DateRangeField} from "@rx-suite/ui";
import {ImportExport} from "@material-ui/icons";
import moment from "moment/moment";
import {DataStore, errToString, Model} from "@rx-suite/data";
import FileSaver from "file-saver";
import {t} from "@lingui/macro";
import {useState} from "react";
import {useSnackbar} from "notistack";
import {makeStyles} from "@material-ui/core/styles";
import {MachineModel} from "../../stores/MachineStore";

type MachineDataViewExportProps = {
    machine: MachineModel;
    store: DataStore<Model>;
    exportedFileName?: string;
    exportEndpoint?: string;
}

export type CSVExport = {
    csv: string;
}

export type CSVError = {
    error: string;
};

const useStyles = makeStyles((theme) => ({
    dialog: {
        margin: 0,
        padding: theme.spacing(2)
    },
    dialogContent: {
        padding: theme.spacing(2)
    },
    dialogHeader: {
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center'
    },
    dialogHeaderText: {
        margin: '0 auto'
    },
    exitButton: {
        position: 'absolute',
        top: '5px',
        right: '5px'
    },
    exportBtn: {
        marginTop: theme.spacing(2),
    },
    exportBtnSection: {
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'center',
        alignItems: 'center',
        marginTop: theme.spacing(1),
        marginBottom: theme.spacing(1)
    },
    csvButton: {
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'flex-end'
    },
    progressBar: {
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'center',
        alignItems: 'center',
        marginTop: theme.spacing(1),
        marginBottom: theme.spacing(1)
    },
    generatingText: {
        marginTop: theme.spacing(1)
    }
}));

export const MachineDataViewExport: React.FunctionComponent<MachineDataViewExportProps> = observer(({machine, store, exportedFileName, exportEndpoint}) => {

    const {enqueueSnackbar} = useSnackbar();

    const classes = useStyles();

    const [open, setOpen] = useState<boolean>(false);

    const [startDate, setStartDate] = useState<moment.Moment>(moment().startOf('day'));
    const [endDate, setEndDate] = useState<moment.Moment>(moment().endOf('day'));

    const [generating, setGenerating] = useState<boolean>(false);
    const [exportBtn, setExportBtn] = useState<boolean>(false);
    const [csv, setCSV] = useState<string|null>(null);

    const handleClose = () => {
        if (generating) {
            return; // as long as an export is being generated, do not close the dialog
        }
        setOpen(false);
    };

    const handleExited = () => {
        setStartDate(moment().startOf('day'));
        setEndDate(moment().endOf('day'));
        setGenerating(false);
        setExportBtn(false);
        setCSV(null);
    };

    const generateCSV = async () => {
        try {
            if (!exportEndpoint) {
                throw new Error(t`Export not provided for this data.`);
            }
            setGenerating(true);
            const exportedData = await machine.exportCSV(exportEndpoint, formatToMillisecondUTC(startDate), formatToMillisecondUTC(endDate));

            if ('error' in exportedData) {
                throw new Error(t`Export failed.`);
            }
            setCSV(exportedData.csv);
            setGenerating(false);
            setExportBtn(true);

        } catch (e) {
            handleClose();
            enqueueSnackbar(errToString(e), {variant: 'error'});
        }
    };

    const downloadCSV = () => {
        try {
            if (!csv) {
                throw new Error(t`Export could not be downloaded.`);
            }
            FileSaver.saveAs(new Blob(
                    [csv],
                    {type: "text/csv;charset=utf-8"}),
                fileName()
            );
        } catch(e) {
            handleClose();
            enqueueSnackbar(errToString(e), {variant: 'error'});
        }
    };

    const fileName = () => {
        const base = `${machine.sn}-${formatToISODate(startDate)}-${formatToISODate(endDate)}`
        if(exportedFileName) {
            return `${base}-${exportedFileName}`
        } else {
            return `${base}-exported.csv`
        }
    };

    const formatToISODate = (date: moment.Moment) => {
        return date.format('YYYY_MM_DDTHHmmss');
    };

    const formatToMillisecondUTC = (date: moment.Moment) => {
        return date.utc().format('YYYY-MM-DDTHH:mm:ss.SSSZ');
    };

    const archiveDate = moment(new Date()).subtract(32, 'd');

    const exportMsg = moment(startDate).isSameOrBefore(archiveDate) || moment(endDate).isSameOrBefore(archiveDate)
        ? t`Generating CSV… Data export is in archive. This may take longer than usual to generate.`
        : t`Generating CSV… This may take a few minutes to generate.`
    ;

    return <>
        {!!exportEndpoint &&
            <ButtonField
                onClick={() => { setOpen(true) }}
                text={t`Export`}
                startIcon={<ImportExport/>}
                loading={store.isLoading}
                tooltip={t`Export the contents of the table`}
                disabled={!exportEndpoint}
            />
        }
        <Dialog
            className={classes.dialog}
            open={open}
            onClose={handleClose}
            TransitionProps={{
                onExited: handleExited
            }}
            maxWidth='xs'
            fullWidth
        >
            <DialogTitle className={classes.dialogHeader} disableTypography>
                <Typography variant="h5" className={classes.dialogHeaderText}>
                    {t`Export`}
                </Typography>
                <IconButton
                    className={classes.exitButton}
                    size={"small"}
                    onClick={handleClose}
                >
                    <CloseIcon />
                </IconButton>
            </DialogTitle>
            <DialogContent className={classes.dialogContent} dividers>
                {generating && <div className={classes.progressBar}>
                    <CircularProgress/>
                    <Typography className={classes.generatingText} variant="subtitle2" color="textSecondary">
                        {exportMsg}
                    </Typography>
                </div>}
                {!generating && !exportBtn && <>
                    <DateRangeField
                        startDate={startDate}
                        endDate={endDate}
                        onClose={() => {}}
                        onStartDateChange={(val) => {

                            if (typeof val === "string") return; // input was invalid

                            if (val.isAfter(endDate)) return; // cannot have startDate > endDate

                            setStartDate(val.set({ second: 0, millisecond: 0 }));
                        }}
                        onEndDateChange={(val) => {

                            if (typeof val === "string") return; // input was invalid

                            if (val.isBefore(startDate)) return; // cannot have endDate < startDate

                            setEndDate(val.set({ second: 59, millisecond: 999 }));
                        }}
                    />
                    <div className={classes.csvButton}>
                        <ButtonField
                            onClick={generateCSV}
                            text={t`Export CSV`}
                            startIcon={<ImportExport/>}
                            loading={generating}
                            tooltip={t`Export the contents of the table into a csv`}
                            disabled={!exportEndpoint}
                        />
                    </div>
                </>}
                {!generating && exportBtn && <div className={classes.exportBtnSection}>
                    <Typography variant="h6">
                        {t`CSV Successfully Generated.`}
                    </Typography>
                    <Button
                        className={classes.exportBtn}
                        color="primary"
                        variant="contained"
                        onClick={downloadCSV}
                    >
                        {t`Download CSV`}
                    </Button>
                </div>}
            </DialogContent>
        </Dialog>
    </>;
});