import * as React from "react";
import {observer} from "mobx-react-lite";
import {RangeBarChart} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {BatchRunDatum, useBatchTimelineToolTip} from "../../../hooks/batch_run_overview/useBatchTimelineToolTip";
import {BATCH_RUN_COLORS} from "../../../hooks/batch_run_overview/batchRunUtilities";

type BatchRunSeries = {
    name: string;
    data: BatchRunDatum[]
}

type MachinePerformanceBatchTimelineReportProps = {
    data: any;
    startTime: string;
    endTime: string;
}

export const BatchCoaterOverviewTimeline: React.FunctionComponent<MachinePerformanceBatchTimelineReportProps> = observer(({data, startTime, endTime}) => {
    const datetimeWithoutTZ = (datetime: string) => {
        return new Date(`${datetime.split('-', 3).join("-")}Z`).getTime();
    }

    // We need to parse the start + end times in the y axis to JS dates
    const series = React.useMemo(() => {
        if(!data.series){
            return [];
        }

        return data.series.map((batch_run: BatchRunSeries) => {
            return {
                name: batch_run.name,
                data: batch_run.data.map(datum => {
                    return {
                        x: datum.x,
                        y: datum.y.map(date => datetimeWithoutTZ(date)),
                        fillColor: datum.fillColor
                    }
                })
            }
        });
    }, [data]);

    const buildToolTip = React.useCallback(({series, seriesIndex, dataPointIndex}) => {
        const dataPoint = data.series[seriesIndex].data[dataPointIndex];

        return useBatchTimelineToolTip(dataPoint);
    }, [])

    return(
        <RangeBarChart series={series}
                       options={{
                           title: {
                               text: t`Batch Runs`,
                               align: "center"
                           },
                           subtitle: {
                               text: data.subtitle,
                               style: {
                                   fontSize: '12px',
                                   fontWeight: 'bold'
                               }
                           },
                           chart: {
                               toolbar: {
                                   show: false
                               },
                               zoom: {
                                   enabled: false,
                               }
                           },
                           plotOptions: {
                               bar: {
                                   barHeight: '100%',
                                   horizontal: true,
                                   dataLabels: {
                                       hideOverflowingLabels: false
                                   }
                               }
                           },
                           xaxis: {
                               type: 'datetime',
                               min: datetimeWithoutTZ(startTime),
                               max: datetimeWithoutTZ(endTime)
                           },
                           legend: {
                               position: 'bottom',
                               customLegendItems: ['Finished', 'Aborted', 'Ongoing'],
                               onItemClick: {
                                   toggleDataSeries: false
                               }
                           },
                           fill: {
                               opacity: 1.0
                           },
                           tooltip: {
                               custom: buildToolTip,
                               x: {
                                   format: 'yy-MM-dd HH:mm'
                               }
                           },
                           colors: [
                               BATCH_RUN_COLORS.FINISHED,
                               BATCH_RUN_COLORS.ABORTED,
                               BATCH_RUN_COLORS.ONGOING
                           ]
                       }}
                       height={'100%'}
        />
    )
})