import * as React from 'react';
import {observer} from "mobx-react-lite";
import {makeStyles} from "@material-ui/core/styles";
import {FormProps, PaddedPaper, sendEnqueueSnackbar, WorkstationHelper, WorkstationGenericWrapper} from "@rx-suite/ui";
import {useEffect} from "react";
import {BlockModel, BlockSettings, getBlockByDMC} from "../../stores/BlockStore";
import {Button, Card, Divider, LinearProgress} from "@material-ui/core";
import Grid from "@material-ui/core/Grid";
import Typography from "@material-ui/core/Typography";
import {green, grey, orange, red} from "@material-ui/core/colors";
import {t} from "@lingui/macro";
import {useSnackbar} from "notistack";
import {routerStore} from "@rx-suite/data";
import {getBlockSettings} from "../../stores/BlockReuseSettingStore";
import {OperatorSearchField} from "../../components/generic/OperatorSearchField";

const useStyles = makeStyles((theme) => ({
    searchContainer: {
        marginBottom: theme.spacing(2),
    },
    searchBox: {
        width: '100%'
    },
    cardContainer: {
        width: '70%',
        margin: 'auto'
    },
    cycleCard: {
        margin: theme.spacing(1),
        padding: theme.spacing(2)
    },
    cycleText: {
        color: grey[600]
    },
    divider: {
        margin: theme.spacing(2),
    },
    blockAction: {
        marginLeft: theme.spacing(2),
        marginRight: theme.spacing(2)
    },
    textDanger: {
        color: red[300]
    },
    textWarning: {
        color: orange[300]
    },
    textGood: {
        color: green[300]
    },
    statusText: {
        marginRight: theme.spacing(1)
    },
    infoContainer: {
        paddingLeft: theme.spacing(2)
    }
}));


export const BlockForm: React.FunctionComponent<FormProps> = observer(({model}) => {

    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();

    const [blockSettings, setBlockSettings] = React.useState<BlockSettings | null>(null);
    const [blockModel, setBlockModel] = React.useState<BlockModel | undefined>(model);

    useEffect(() => {
        const getBlockReuseSetting = async () => {
            const settings = await getBlockSettings();
            setBlockSettings(settings);
        };
        getBlockReuseSetting();
    }, []);

    function attemptBlockAction(result: BlockModel|undefined): void {
        if (result) {
            sendEnqueueSnackbar(t`Block Updated Successfully.`, {variant: "success"}, enqueueSnackbar);
            setBlockModel(result);
        } else {
            sendEnqueueSnackbar(t`Error in reaching Rx-Data.`, {variant: "error"}, enqueueSnackbar);
        }
    }

    async function handleBlockSearch(searchFieldValue: string): Promise<void> {
        const model = await getBlockByDMC(searchFieldValue);
        if (model) {
            sendEnqueueSnackbar(t`Block Found.`, {variant: "success"}, enqueueSnackbar);
            routerStore.history.push(`/blocks/${model['id']}/edit`);
            setBlockModel(model);
        } else {
            sendEnqueueSnackbar(t`No Block matching DMC found.`, {variant: "error"}, enqueueSnackbar);
            routerStore.history.push(`/blocks/search`);
            setBlockModel(undefined);
        }
    }

    function colorForValue(current: number, max: number) {
        const usage = (max === 0 ? 0 : current / max);
        if (usage > 0.60) {
            return classes.textGood;
        }
        if (usage > 0.25) {
            return classes.textWarning;
        }
        return classes.textDanger;
    }

    if (!blockSettings) {
        return <LinearProgress color={'secondary'} />;
    }

    let blockInfo = undefined;
    if (blockModel) {
        const block = blockModel as BlockModel;
        let status: string = t`Good`;
        let statusClass = classes.textGood;
        if (block.cycles_until_wash as number <= 0) {
            status = t`Wash Required`;
            statusClass = classes.textWarning;
        }
        if (block.cycles_until_discard as number <= 0) {

            status = t`Discard`;
            statusClass = classes.textDanger;
        }
        let blockActions;
        if (block.cycles_until_discard > 0) {
            blockActions = <Grid container className={classes.cardContainer} justify="center">
                <Button className={classes.blockAction} size="large" variant="contained" color="primary" disableElevation onClick={async ()=>{attemptBlockAction(await block.wash())}}>{t`Wash Block`}</Button>
                {block.cycles_until_wash > 0 && <Button className={classes.blockAction} size="large" variant="contained" color="primary" disableElevation onClick={async ()=>{attemptBlockAction(await block.forceToClean())}}>{t`Force To Clean`}</Button>}
                <Button className={classes.blockAction} size="large" variant="contained" color="primary" disableElevation onClick={async ()=>{attemptBlockAction(await block.discard())}}>{t`Discard Block`}</Button>
            </Grid>;
        } else {
            blockActions = <Grid container className={classes.cardContainer} justify="center">
                <Button className={classes.blockAction} size="large" variant="contained" color="primary" disableElevation onClick={async ()=>{attemptBlockAction(await block.reactivate())}}>{t`Reactivate`}</Button>
            </Grid>;
        }
        blockInfo = <WorkstationGenericWrapper>
            <Grid container className={classes.cardContainer} justify="space-around">
                <Grid item>
                    <Card className={classes.cycleCard}>
                        <Grid container direction="column" alignItems="center">
                            <Typography variant={'h5'}>{t`Block Use Count`}</Typography>
                            <Typography variant={'h3'}>{block.usage_count}</Typography>
                            <Typography className={classes.cycleText} variant={'h5'}>{t`Cycles`}</Typography>
                        </Grid>
                    </Card>
                </Grid>
                <Grid item>
                    <Card className={classes.cycleCard}>
                        <Grid container direction="column" alignItems="center">
                            <Typography variant={'h5'}>{t`Wash Block In`}</Typography>
                            <Typography className={colorForValue(block.cycles_until_wash, blockSettings.block_wash_after)} variant={'h3'}>{block.cycles_until_wash}</Typography>
                            <Typography className={classes.cycleText} variant={'h5'}>{t`Cycles`}</Typography>
                        </Grid>
                    </Card>
                </Grid>
                <Grid item>
                    <Card className={classes.cycleCard}>
                        <Grid container direction="column" alignItems="center">
                            <Typography variant={'h5'}>{t`Discard Block In`}</Typography>
                            <Typography className={colorForValue(block.cycles_until_discard, blockSettings.block_discard_after)} variant={'h3'}>{block.cycles_until_discard}</Typography>
                            <Typography className={classes.cycleText} variant={'h5'}>{t`Cycles`}</Typography>
                        </Grid>
                    </Card>
                </Grid>
            </Grid>
            <Grid container justify="center">
                <Typography className={classes.statusText} variant={'h5'}>{t`Status`}:</Typography><Typography className={statusClass} variant={'h5'}>{`${status}`}</Typography>
            </Grid>
            <Divider className={classes.divider}/>
            {blockActions}
            <Divider className={classes.divider}/>
            <div className={classes.infoContainer}>
                <Typography variant={'h4'}>{t`Block Information`}</Typography>
                <Typography variant={'h6'}>DMC: {block.dmc}</Typography>
                <Typography variant={'h6'}>{t`Last Used By Job`}: {block.job_number}</Typography>
                <Typography variant={'h6'}>{t`Job Side`}: {block.job_side}</Typography>
                <Typography variant={'h6'}>{t`Radius`}: {block.dmc.substring(0,3)}mm</Typography>
                <Typography variant={'h6'}>{t`Diameter`}: {block.dmc.substring(3,5)}mm</Typography>
                <Typography variant={'h6'}>{t`Height`}: {block.dmc.substring(5,7)}mm</Typography>
            </div>
        </WorkstationGenericWrapper>;
    }

    return (
        <WorkstationHelper stores={[]}>
            <Grid container className={classes.searchContainer}>
                <Grid item className={classes.searchBox}>
                    <OperatorSearchField searchFieldPlaceHolderText={t`Enter or Scan Block DMC`} searchFunction={handleBlockSearch}/>
                </Grid>
            </Grid>
            {blockInfo !== undefined && <PaddedPaper>
                {blockInfo}
            </PaddedPaper>}
        </WorkstationHelper>
    );
});