import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";

@model({
    transportLayer: "/api/blocks",
    permissionTarget: "BlockReuse",
    routingInfo: {
        read: '/blocks/:id/edit',
        update: '/blocks/:id/edit'
    }
})
export class BlockModel extends Model {
    @attribute({
        label: t`DMC`,
        validation: yup.string()
    })
    dmc: string = '';

    @attribute({
        label: t`Side`,
        validation: yup.string().required()
    })
    side: string = '';

    @attribute({
        label: t`RFID tag`,
        validation: yup.string().nullable()
    })
    rfid_tag: string = '';

    @attribute({
        label: t`Usage Count`,
        validation: yup.number().required()
    })
    usage_count: number = 0;

    @attribute({
        label: t`Cycles Until Wash`,
        validation: yup.number().required()
    })
    cycles_until_wash: number = 0;

    @attribute({
        label: t`Cycles Until Discard`,
        validation: yup.number().required()
    })
    cycles_until_discard: number = 0;

    @attribute({
        label: t`Last Job Number`,
        validation: yup.string()
    })
    job_number: string = '';

    @attribute({
        label: t`Job Side`,
        validation: yup.string()
    })
    job_side: string = '';

    @attribute({
        label: t`Created at`,
        validation: yup.string()
    })
    created_at: string = '';

    @attribute({
        label: t`Created at`,
        validation: yup.string()
    })
    updated_at: string = '';

    async reactivate(): Promise<BlockModel|undefined> {
        return await this.attemptBlockAction('reactivate');
    }

    async discard(): Promise<BlockModel|undefined> {
       return await this.attemptBlockAction('discard');
    }

    async wash(): Promise<BlockModel|undefined> {
        return await this.attemptBlockAction('wash');
    }

    async forceToClean(): Promise<BlockModel|undefined> {
        return await this.attemptBlockAction('force_to_clean');
    }

    private async attemptBlockAction(action: string): Promise<BlockModel|undefined> {
        try {
            const response = await BlockStore.transportLayer.fetch(`/${this.id}/${action}`, {method: 'put'});

            const data = await response.json();

            const model = this.store.createModel(data) as BlockModel;

            this.store.setModel(model);

            return model;
        } catch (e) {
            return undefined;
        }
    }

    constructor() {
        super();
        super.setup();
    }
}

export async function getBlockByDMC(dmc: string): Promise<BlockModel|undefined> {
    try {
        const response = await BlockStore.transportLayer.fetch(`/find_by_dmc?dmc=${dmc}`);

        const data = await response.json();

        const block = BlockStore.createModel(data);

        BlockStore.setModel(block);

        return block;
    } catch (e) {
        return undefined;
    }
}

export const BlockStore = BlockModel.store as DataStore<BlockModel>;

export type BlockSettings = {
    block_wash_after: number;
    block_discard_after: number;
    enable_server: boolean;
}