module KpiTableHandler
  module Base
    module KpiCacheHelpers
      extend ActiveSupport::Concern
      include ArchiveHelpers

      # Several methods used by KpiTableHandler::Base to interact with the KPI cache

      def all_cache_entries_exist?(from_sub_query, start_time, end_time)
        required_cache_entry_count = (((end_time - start_time) / KpiHelpers.cache_entry_size).ceil) * machine_ids.size
        from_sub_query.unscope(:group).unscope(:select).count == required_cache_entry_count
      end

      def find_and_build_missing_cache_entries(from_sub_query, start_time, end_time)
        # We're missing some entries, find which is missing
        missing_cache_entries = []

        query = find_missing_timeframes_query(from_sub_query, start_time, end_time)

        # Build missing cache entry for each machine individually
        machine_ids.each do |machine_id|
          missing_cache_entries.concat(find_missing_cache_entries_for_machine(query, machine_id, start_time, end_time))
        end

        # Iterate over missing cache times and call the kpi class's backward cache calculation
        cache_entries_to_insert = build_missing_cache_entries(missing_cache_entries)

        # Save the missing cache entry
        insert_cache_entries(table, cache_entries_to_insert) unless cache_entries_to_insert.empty?
      end

      def find_missing_timeframes_query(from_sub_query, start_time, end_time)
        from_sub_query.unscope(:group)
                      .unscope(:select)
                      .where(kpi_table_class.arel_table[:time].gteq(start_time)
                                            .and(kpi_table_class.arel_table[:time].lt(end_time)))
                      .order(:time)
      end


      def find_missing_cache_entries_for_machine(query, machine_id, start_time, end_time)
        time = start_time
        missing_cache_entries = []

        KpiHelpers.get_time_ordered_batches(query.where(table.arel_table[:machine_id].eq(machine_id))) do |batch|
          batch.each do |kpi|
            while time < kpi.time
              missing_cache_entries << {time: time, machine_id: machine_id}
              time += 30.minutes
            end
            time += 30.minutes
          end
        end

        # Find the entries missing after the last bucket and the end time
        while time < end_time
          missing_cache_entries << {time: time, machine_id: machine_id}
          time += 30.minutes
        end

        missing_cache_entries
      end

      def build_missing_cache_entries(missing_entries)
        cache_entries_to_insert = []
        missing_entries.each do |entry|
            # Dont create cache entries for stuff that's in the future or past 32.days ago
            unless (entry[:time] > Time.zone.now) || !time_exists_in_primary?(entry[:time])
              hash_arr = kpi_table_class.backwards_cache_entry_calculation(entry)
              # Returns an array of hash, each hash being a column to be inserted in the kpi cache table
              cache_entries_to_insert.concat hash_arr
            end
        end
        cache_entries_to_insert
      end

      def insert_cache_entries(kpi_table, cache_entries)
        begin
          until cache_entries.empty?
            # PG cannot insert 1000 records or more at once, so we're limited to 999
            cache_to_insert = cache_entries.shift(999)
            kpi_table.insert_all!(cache_to_insert)
          end
        rescue StandardError => err
          # Failed writing the cache, likely a recently added entry breaks the index, just ignore
          Rails::logger.warn "Failed to insert all cache records for table #{kpi_table_class.to_s}. error #{err}"
        end
      end
    end
  end
end