module KpiTableHandler
  module IntervalBuilder
    extend ActiveSupport::Concern

    include ArchiveHelpers
    include KpiHelpers

    def aggregate_machines?
      !!options[:aggregate_machines]
    end

    def result_builder(start_time, end_time, &block)
      @intervals = build_time_intervals(start_time, end_time)
      @results = {}

      append_to_results_helper(build_archive_results(&block)) unless @intervals[:archive].nil?

      append_to_results_helper(build_now_results(&block)) unless @intervals[:now].nil?

      append_to_results_helper(build_future_results(&block)) unless @intervals[:future].nil?

      @results
    end

    def build_archive_results(&block)
      ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
        block.call(*interval_start_end_time(:archive))
      end
    end

    def build_now_results(&block)
      block.call(*interval_start_end_time(:now))
    end

    def build_future_results(&block)
      block.call(*interval_start_end_time(:future))
    end

    def interval_start_end_time(target)
      [@intervals[target][:bucket_start_time], @intervals[target][:bucket_end_time]]
    end

    def build_time_intervals(start_time, end_time)
      raise StartTimeAfterEndTime if end_time < start_time
      bucket_start_time = KpiHelpers.kpi_bucket_time(start_time)
      # KpiHelpers.kpi_bucket_time(end_time) will give the start time of the end_bucket, but we want to end time of that bucket.
      bucket_end_time = KpiHelpers.kpi_bucket_time(end_time).advance(seconds: KpiHelpers.cache_entry_size_seconds - 1.seconds)
      {
        now: get_now_intervals(bucket_start_time, bucket_end_time),
        future: get_future_intervals(bucket_start_time, bucket_end_time),
        archive: get_archive_intervals(bucket_start_time, bucket_end_time)
      }
    end

    # Includes buckets within 1 day of data_expired_date (buffer)
    def get_archive_intervals(bucket_start_time, bucket_end_time)
      # Now calculation has the bucket at KpiHelpers.kpi_bucket_time(data_expired_date + 1.days), so we want to buket right before
      last_archive_bucket_time = KpiHelpers.kpi_bucket_time(data_expired_date + 1.days) - KpiHelpers::cache_entry_size
      return nil if bucket_start_time > last_archive_bucket_time
      {
        bucket_start_time: bucket_start_time,
        bucket_end_time: [bucket_end_time, last_archive_bucket_time].min
      }
    end

    # Excludes buckets within 1 day of data_expired_date (buffer)
    # Includes "current" bucket
    def get_now_intervals(bucket_start_time, bucket_end_time)
      first_now_bucket_time = KpiHelpers.kpi_bucket_time(data_expired_date + 1.days)
      return nil if bucket_end_time < first_now_bucket_time
      last_now_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now + KpiHelpers.cache_entry_size)
      return nil if bucket_start_time > last_now_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_now_bucket_time].max,
        bucket_end_time: [bucket_end_time, last_now_bucket_time].min
      }
    end

    # Excludes "current" bucket
    def get_future_intervals(bucket_start_time, bucket_end_time)
      first_future_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now + KpiHelpers.cache_entry_size)
      return nil if bucket_end_time < first_future_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_future_bucket_time].max,
        bucket_end_time: bucket_end_time
      }
    end

    def append_to_results_helper(data)
      if timeframe == KpiHelpers::Timeframe::ALL
        append_to_result_all(data)
      else
        append_to_results(data)
      end
    end

    def append_to_results(data)
      if aggregate_machines?
        return @results.merge!(data) {|k, a_val, b_val| a_val.merge(b_val) {|k, a, b| a + b} }
      end

      data.each do |machine_id, buckets|
        @results[machine_id] = {} if @results[machine_id].nil?
        @results[machine_id].merge!(buckets) {|k, a_val, b_val| a_val.merge(b_val) {|k, a, b| a + b} }
      end
    end

    def append_to_result_all(data)
      if aggregate_machines?
        @results.merge!(data) {|k, a, b| a + b }
      else
        data.each do |machine_id, buckets|
          @results[machine_id] ||= {}
          @results[machine_id].merge!(buckets) {|k, a, b| a + b }
        end
      end
    end
  end
end