module KpiTableHandler
  class MachineStateDurationKpiHandler < KpiTableHandler::Base::KpiTableHandler
    include KpiHelpers

    class MachineStateNotSpecified < StandardError; end

    attr_accessor :machine_states, :sum_as, :force_as_percent

    def initialize(start_time, end_time, timeframe, machine_ids, params: {}, options: {})
      raise MachineStateNotSpecified unless params.key?(:machine_states)

      self.machine_states = params[:machine_states]
      self.sum_as = params[:sum_as]
      # If params indicate to use a percent, ignore the global options
      self.force_as_percent = !!params[:as_percent] || false

      super(start_time, end_time, timeframe, machine_ids, options: options)
    end

    def table
      KpiMachineStateDuration
    end

    def base_select
      select = machine_states.map do |state|
        # Have to wrap sum as string with "" because states like "on" and "off" are reserved sql keywords
        table.arel_table[state.to_sym].sum.as("\"#{state.to_s}\"")
      end

      #TODO: probably add as base to kpi handler?
      select << table.arel_table[:machine_id].as("machine_id") unless aggregate_machines?


      select
    end

    def kpi_to_extract
      query = if as_sum?
                "(#{machine_states.join(" + ")}) as \"#{sum_as}\""
              else
                machine_states.map {|m| "\"#{m.to_s}\""}.join(", ")
              end

      #TODO should probably be generic?
      query += ", machine_id" unless aggregate_machines?

      query
    end

    def timeframe_select
      selects = []
      if as_sum?
        selects << "SUM(#{sum_as}) as \"#{sum_as}\""
        selects << table.arel_table[:machine_id] unless aggregate_machines?
      else
        selects.concat(base_select)
      end
      selects
    end

    def timeframe_group
      base_group
    end

    def col_name
      cols = []
      if as_sum?
        cols << sum_as
      else
        cols.concat(machine_states.map {|state| state.to_sym})
      end

      #TODO: should be in base KPI?
      cols << :machine_id unless aggregate_machines?
      cols
    end

    def run
      results = super
      format_results(results)
    end

    # record is still an active record at this point, so extract the values we want
    def record_value(record)
      result = {}

      if as_sum?
         result[sum_as.to_sym] = record[sum_as.to_sym]
      else
        machine_states.each do |state|
          result[state.to_sym] = record[state.to_sym]
        end
      end
      result
    end

    def as_sum?
      return false if sum_as.nil?
      true
    end

    def as_percent?
      return true if force_as_percent
      return false if options.nil? || options[:as_percent].nil?
      options[:as_percent]
    end

    def format_results(results)
      return results unless as_percent?

      if aggregate_machines?
        return format_buckets(results, machine_count)
      else
        results.each do |machine_id, buckets|
          results[machine_id] = format_non_aggregated_buckets(buckets)
        end
        results
      end
    end

    def format_non_aggregated_buckets(row)
      format_buckets(row, 1)
    end

    def format_buckets(row, m_count)
      if timeframe == Timeframe::ALL
        format_single_bucket(row, m_count)
      else
        row.each do |time, col|
          bucket_size = bucket_size_for_timeframe(time)
          col.each do |ck, cv|
            row[time][ck] = ((cv / (bucket_size * m_count)) * 100).to_f.round(2)
          end
        end
      end
    end

    def format_single_bucket(row, m_count)
      bucket_size = bucket_size_for_timeframe(nil)
      row.each do |ck, cv|
        row[ck] = ((cv / (bucket_size * m_count)) * 100).to_f.round(2)
      end
    end

    # If the requested timeframe is 'now' we have to truncate the timeframe to the nearest bucket and add the current bucket size
    def bucket_size_for_timeframe(time)
      # In the case where our bucket start time is earlier then the first state ever of the machine, we consider time spent before this state to be unknown,
      # so we use the bucket timeframe to calculate the percentage anyways. Example: time=00:00, processing at 12:00 with nothing before,
      # percentage of processing will still be 50%
      case timeframe
      when KpiHelpers::Timeframe::HOUR_BY_HOUR
        if time + 1.hour > Time.zone.now
          #TODO: test this case
          now_bucket_size        # Special case: hourly buckets may require an extra 30 min offset depending the current time
        else
          # AFAIK 1 and offset are the most granular time type, so cant have anything else than 1.hour bucket time
          return 1.hour
        end
      when KpiHelpers::Timeframe::DAY_BY_DAY
        # only activate if bucket day is same or greater than time now and end time is greater than time now bucket
        if time >= Time.zone.now.beginning_of_day && end_time >= KpiHelpers::kpi_bucket_time(Time.zone.now)
          return (KpiHelpers::kpi_bucket_time(Time.zone.now) - time.beginning_of_day) + KpiHelpers.machine_state_duration_current_bucket_size
        else
          # day by day will always span one day at a time unless its the now bucket, which is dealt above
          # We need to calculate actual time spent in bucket and not just bucket length
          if time < start_time
            # from midnight to bucket start is excluded
            1.day - start_time.seconds_since_midnight
          elsif time.day == end_time.day
            # Time spent in bucket
            end_time.seconds_since_midnight
          else
            1.day
          end
        end
      when KpiHelpers::Timeframe::WEEK_BY_WEEK
        # week by week will always span one week at a time unless its the now bucket
        if time.beginning_of_week(:sunday) == Time.zone.now.beginning_of_week(:sunday)
          return (KpiHelpers::kpi_bucket_time(Time.zone.now) - time.beginning_of_week(:sunday)) + KpiHelpers.machine_state_duration_current_bucket_size
        else
          return 1.week
        end
      when KpiHelpers::Timeframe::ALL
        if end_time > Time.zone.now
          return Time.zone.now - start_time
        else
          return end_time - start_time
        end
      end
    end

    def now_bucket_size
      offset = if now_bucket_requires_offset?
        KpiHelpers.cache_entry_size
      else
        0
      end

      KpiHelpers.machine_state_duration_current_bucket_size + offset
    end

    # If our start time is at the mid-hour (say 4h30)
    #   if it is currently 4h31 we don't need an offset (we are in the first half of the 'hour bucket').
    #   if it is currently 5h01 we need an offset (we are in the second half of the 'hour bucket').
    # If our start time is at the start of the hour (say 4h00)
    #    if it is currently 4h01 we don't need an offset (we are in the first half of the 'hour bucket').
    #    if it is currently 4h31 we need an offset (we are in the second half of the 'hour bucket').
    def now_bucket_requires_offset?
      return true if KpiHelpers.time_is_mid_hour(start_time) && !KpiHelpers.time_is_in_second_half_hour?(Time.zone.now)
      KpiHelpers.time_is_in_second_half_hour?(Time.zone.now)
    end

    def hash_targets_col
      machine_states.map{|m| m.to_sym}
    end

    def apply_hash_kpi_operation(hash)
      hash = hash_sum_col(hash)

      hash
    end

    # For this KPI we want to treat the 'now bucket' as a future bucket
    def get_now_intervals(bucket_start_time, bucket_end_time)
      first_now_bucket_time = KpiHelpers.kpi_bucket_time(data_expired_date + 1.days)
      return nil if bucket_end_time < first_now_bucket_time
      last_now_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now)
      return nil if bucket_start_time > last_now_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_now_bucket_time].max,
        bucket_end_time: [bucket_end_time, last_now_bucket_time].min
      }
    end

    def get_future_intervals(bucket_start_time, bucket_end_time)
      first_future_bucket_time = KpiHelpers.kpi_bucket_time(Time.zone.now)
      return nil if bucket_end_time < first_future_bucket_time
      {
        bucket_start_time: [bucket_start_time, first_future_bucket_time].max,
        bucket_end_time: bucket_end_time
      }
    end

    private

    # Sum Key values and remap to new key
    def hash_sum_col(hash)
      return hash unless as_sum?

      if aggregate_machines?
        hash = buckets_sum_as_transformation(hash)
      else
        hash.each do |machine_id, buckets|
          hash[machine_id] = buckets_sum_as_transformation(buckets)
        end
      end

      hash
    end

    def buckets_sum_as_transformation(buckets)
      buckets.each do |time_bucket, data|
        buckets[time_bucket] = {sum_as.to_sym => data.values.sum}
      end
      buckets
    end
  end
end