module MachineKpiCalculator
  module BoxCoaterBatches
    class BatchRuns < MachineKpiCalculator

      include KpiHelpers
      include KpiTableHandler
      include IntervalBuilder

      class << self
        def get_for_period_and_machines(start_time, end_time, machine_ids, options: {})
          # Due to the nature of batch durations, it is impossible to bucket them by timeframe, so it's always going to be the entire period
          kpi = BatchRuns.new(start_time, end_time, KpiHelpers::Timeframe::ALL, machine_ids, options)
          kpi.run
        end
      end

      def initialize(start_time, end_time, timeframe, machine_ids, options)
        super(start_time, end_time, timeframe, machine_ids, options)
      end

      def handlers
        [
          {
            key: :batch_runs,
            handler: :batch_runs,
            params: {}
          }
        ]
      end

      def batch_runs
        result_builder(start_time, end_time) do |interval_start, interval_end|
          interval_results = {}
          machine_ids.each do |machine_id|
            interval_results[machine_id] = interval_batch_runs(machine_id, interval_start, interval_end)
          end
          interval_results
        end
      end

      def interval_batch_runs(machine_id, interval_start, interval_end)
        query = JobState.where(
          time: interval_start..interval_end,
          machine_id: machine_id
        ).order(time: :asc)

        job_state_durations_in_order = []

        previous_state = nil

        KpiHelpers.get_time_ordered_batches(query) do |job_states|
          job_states.each do |job_state|
            # EDGE CASE 1: first state is 'finished' or 'aborted'
            # If the first job state we encounter is 'finished' or 'aborted', the start state could be in another interval
            if previous_state.nil? && job_state.is_terminal?
              # Only check for a previous started if we're at the very beginning of the report, otherwise the edge case for ending with started will find this job_state
              if start_time.to_i == interval_start.to_i
                previous_state = find_previous_start_state(job_state)
              end
            end

            # If somehow we have a terminal state but no starting state, ignore this job state
            # Likewise if this is the first started, we cannot calculate a duration yet
            if (job_state.is_terminal? && previous_state.nil?) || (job_state.is_started? && previous_state.nil?)
              previous_state = job_state
              next
            end

            if previous_state.started? && job_state.is_terminal?
              job_state_durations_in_order << build_batch_run(previous_state.time, job_state.time, job_state.state)
            end

            previous_state = job_state
          end
        end

        unless previous_state.nil?
          # EDGE CASE 2: the last state is started, the batch is completed in another interval or is ongoing
          if previous_state.is_started?
            next_state = find_next_terminal_state(previous_state)

            if next_state.nil?
              job_state_durations_in_order << build_batch_run(previous_state.time, Time.now, 'ongoing')
            else
              job_state_durations_in_order << build_batch_run(previous_state.time, next_state.time, next_state.state)
            end
          end
        end

        job_state_durations_in_order
      end

      def build_batch_run(start_time, end_time, state)
        {
          start: start_time,
          end: end_time,
          state: state,
          duration: end_time - start_time
        }
      end

      def find_previous_start_state(job_state)
        JobState.where(machine_id: job_state.machine_id, job_id: job_state.job_id, time: ...job_state.time, state: "started")
                .order(time: :desc)
                .first
      end

      def find_next_terminal_state(job_state)
        JobState.where(machine_id: job_state.machine_id, job_id: job_state.job_id, time: job_state.time..., state: %w[finished aborted])
                .order(time: :asc)
                .first
      end

      def append_to_result_all(interval_data)
        interval_data.each do |machine_id, batches|
          @results[machine_id] = [] if @results[machine_id].nil?
          @results[machine_id] += batches
        end
      end

      def build_results
        # Probably should return a flat array of all batches over the duration ? Not sure if it's useful however
        raise NotImplementedError
      end

      def build_results_per_machine
        data[:batch_runs]
      end
    end
  end
end