module MachineKpiCalculator
  class EventDrlFinishedData < MachineKpiCalculator

    include KpiTableHandler::IntervalBuilder

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        kpi = EventDrlFinishedData.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [{
        key: :drl_finished_events,
        handler: :drl_finished_event_data,
        params: {}
      }]
    end

    def machine_sns
      Machine.where(id: machine_ids).pluck(:sn)
    end

    def event_cols
      [:message, :machine_sn, :time]
    end

    def drl_finished_event_data
      result_builder(start_time, end_time) do |s_time, e_time|
        query = query_builder(s_time, e_time)
        load_results(query, s_time, e_time)
      end
    end

    def query_builder(start_time, end_time)
      # Instead of using pivots, we will use conditional aggregation to speed up calculations.
      # Note on this implementation, if you have multiple events with same message, time, and sn, it'll be grouped into one
      # Generates this SQL:
      # SELECT [events].[message], [events].[machine_sn], [events].[time],
      # tx = MAX(IIF([event_data].[name] = N'Tx', [event_data].[value], null)),
      # ty = MAX(IIF([event_data].[name] = N'Ty', [event_data].[value], null)),
      # rz = MAX(IIF([event_data].[name] = N'Rz', [event_data].[value], null)),
      # time_started = MAX(IIF([event_data].[name] = N'Time Started', [event_data].[value], null)),
      # time_finished = MAX(IIF([event_data].[name] = N'Time Started', [event_data].[value], null)),
      # result = MAX(IIF([event_data].[name] = N'Result', [event_data].[value], null)),
      # status = MAX(IIF([event_data].[name] = N'Status', [event_data].[value], null))
      # FROM [events] INNER JOIN [event_data]
      # ON [event_data].[event_id] = [events].[id]
      # GROUP BY [events].[message], [events].[time], [events].[machine_sn]

      # Columns needed for DRL from EventData
      event_data_cols = ['Tx', 'Ty', 'Rz', 'Time Started', 'Time Finished', 'Result', 'Status']

      # Method to that generates this:
      # IIF([event_data].[name] = N'Tx', [event_data].[value], null)
      ternary_sql = lambda do |condition, result_true, result_false|
        Arel::Nodes::NamedFunction.new('IIF', [condition, result_true, result_false])
      end

      # Method to that generates this:
      # tx = MAX(ternary_sql from above)
      conditionally_aggregated_column = lambda do |column_name|
        Arel.sql(column_name.gsub(" ", "_").downcase).eq(
          ternary_sql.call(
            EventDatum.arel_table[:name].eq(column_name),
            EventDatum.arel_table[:value],
            Arel.sql('null')
          ).maximum
        )
      end

      # Generate a conditional_aggregation for each column needed from event_data
      conditionally_aggregated_columns = event_data_cols.map do |column_name|
        conditionally_aggregated_column.call(column_name)
      end

      Event.select(event_cols + conditionally_aggregated_columns)
           .joins(:event_data)
           .group(event_cols)
           .where(time: start_time..end_time)
           .where(message: 'DRL_finished')
           .where(Event.arel_table[:machine_sn].in(machine_sns))
           .order(:time)
    end

    def count_query(start_time, end_time)
      # Since we do count of the get time ordered batches doesnt work with our groupings, so we get the id of the events and then count that
      event_ids = Event.select(event_cols + [Event.arel_table[:id].minimum.as('id')])
                    .where(time: start_time..end_time)
                    .where(message: 'DRL_finished')
                    .where(Event.arel_table[:machine_sn].in(machine_sns))
                    .group(event_cols + [:id])
                    .order(:time).pluck(:id)

      Event.where(Event.arel_table[:id].in(event_ids))
    end

    def load_results(query, start_time, end_time)
      hash = {}
      KpiHelpers.get_time_ordered_batches(query, count_query: count_query(start_time, end_time)) do |batch|
        batch.each do |event|
          hash[event[:machine_sn]] ||= []
          # Load event attributes
          hash[event[:machine_sn]].push(event.as_json.symbolize_keys)
        end
      end
      hash
    end

    def append_to_results(data)
      data.each do |machine_id, buckets|
        @results[machine_id] ||= []
        @results[machine_id] += buckets
      end
    end

    def build_results
      data[:drl_finished_events]
    end

    def build_results_per_machine
      data[:drl_finished_events]
    end
  end
end