module MachineKpiCalculator

  # Base class for all Machine KPIs
  class MachineKpiCalculator

    class InvalidKpiHandler < StandardError; end

    include KpiHelpers
    attr_accessor :start_time, :end_time, :timeframe, :machine_ids, :options, :data

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        raise NotImplementedError
      end
    end

    def force_all(options)
      options[:force_all]
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      self.start_time = start_time
      self.end_time = end_time
      self.timeframe = timeframe
      self.machine_ids = machine_ids
      self.options = options
      self.data = {}
    end

    def aggregate_machines?
      !!options[:aggregate_machines]
    end

    def sum_as
      options[:sum_as]
    end

    def run
      build_data

      return build_results if aggregate_machines?

      build_results_per_machine
    end

    private

    # Handles calling the the handlers one at a time and dumping the result of each handler in data
    def build_data
      handlers.each do |handler|
        to_call = handler[:handler]
        key = handler[:key]

        if to_call.is_a?(Class) && to_call.method_defined?(:kpi_table_class)
          # to_call is a KPI handler, run it now
          data[key] = to_call.new(start_time, end_time, timeframe, machine_ids, options: options, params: handler[:params]).run
        elsif  to_call.is_a? Symbol
          data[key] = method(to_call).call
        else
          raise InvalidKpiHandler
        end
      end
    end

    # Array of KPI handlers or method to call to build the data required to build the result.
    # The key of the handler is how it will be saved in the data accumulator.
    # The params will be passed as-is to the KPI handler
    #
    # The handler can be a KpiHandler or the symbolized name of a method defined in the KPI itself
    # ex:
    # [
    #     {key: :downtime, handler: DowntimeKpiHandler, params: {:machine_states: ['estop', 'halt', 'error']}},
    #     {key: :number_failure, handler: :find_number_of_failures}
    # ]
    def handlers
      raise NotImplementedError
    end

    def handlers_per_machine
      raise NotImplementedError
    end

    # This method should use the data built by the handlers and return the final result of the KPI
    def build_results
      raise NotImplementedError
    end

    def build_results_per_machine
      raise NotImplementedError
    end
  end
end