# == Schema Information
#
# Table name: batches
#
#  id           :bigint           not null, primary key
#  process_code :string(4000)
#  created_at   :datetime         not null
#  updated_at   :datetime         not null
#  batch_id     :string(4000)
#
# Indexes
#
#  index_batches_on_batch_id  (batch_id) UNIQUE
#
class Batch < ApplicationRecord
  has_many :batch_runs
  has_many :batch_containers

  validates :batch_id, uniqueness: { message: '- Batch ID is already in use.' }

  class MissingBatchData < StandardError; end

  def self.save_batch(json_data)
    raise MissingBatchData if json_data.nil? || json_data[:batchData].nil? || json_data[:batchData][:sectorData].nil?
    batch_id = nil
    ActiveRecord::Base.transaction do
      batch_data = json_data[:batchData]
      sector_data = batch_data[:sectorData]
      runs = batch_data[:runs]
      batch = Batch.new({
                          batch_id: json_data[:batchId],
                          process_code: json_data[:processCode]
                        })
      batch.batch_runs.build(self.build_batch_runs_for_runs(runs))
      batch.batch_containers.build(self.build_batch_containers_for_sector_data(sector_data))
      batch.save! # save the batch to create both the batch_containers and batch_run, needed for deeper associations
      batch.batch_runs.each_with_index do |batch_run, i|
        found_run = runs.find { |run| run[:run_key] === batch_run.run_key }
        next if found_run.nil?
        batch.batch_runs[i].batch_run_data.build(self.build_run_data_for_run(found_run))
        batch.batch_runs[i].save!
      end
      batch_containers = batch.batch_containers.order(id: :asc)
      sector_data.each_with_index do |sector_datum, i|
        build_batch_container_data = self.build_batch_container_data_for_sector_datum(sector_datum)
        batch_containers[i].batch_container_data.build(build_batch_container_data)
        build_batch_job_data = self.build_batch_job_data_for_sector_datum(sector_datum)
        batch_containers[i].batch_job_data.build(build_batch_job_data) if !build_batch_job_data.nil?
        batch_containers[i].save!
      end
      batch_id = batch.id
    end
    batch_id
  end

  def get_json
    {
      :processCode => self.process_code,
      :batchId => self.batch_id,
      :createdAt => self.created_at,
      :batchData => {
        :runs => build_runs_for_batch_runs(self.batch_runs),
        :sectorData => build_sector_data_for_batch_containers(self.batch_containers)
      }
    }
  end

  def self.delete_before(date)
    deleted_objects = 0
    # Delete the batches that have existed before the given date
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      batches = Batch.where(Batch.arel_table[:created_at].lt(date))
      deleted_objects += batches.delete_all
    end
    deleted_objects
  end

  private

  # JSON builder methods
  def build_runs_for_batch_runs(batch_runs)
    batch_runs.map do |batch_run|
      {
        :run_key => batch_run[:run_key]
      }.merge(build_run_data_for_batch_run(batch_run))
    end
  end

  def build_run_data_for_batch_run(batch_run)
    batch_run.batch_run_data.map do |batch_run_datum|
      [batch_run_datum.name.to_sym, batch_run_datum.value]
    end.to_h
  end

  def build_sector_data_for_batch_containers(batch_containers)
    batch_containers.order(order: :asc).map do |bc|
      {
        :containerId => bc[:external_container_id],
        :order => bc[:order],
        :lensData => build_lens_data_for_batch_container(bc)
      }.merge(build_batch_container_data_for_batch_container(bc))
    end
  end

  def build_batch_container_data_for_batch_container(batch_container)
    batch_container.batch_container_data.map do |batch_container_datum|
      [batch_container_datum.name.to_sym, batch_container_datum.value]
    end.to_h
  end

  def build_lens_data_for_batch_container(batch_container)
    batch_container.batch_job_data.order(position: :asc).map do |bjd|
      {
        :position => bjd[:position],
        :jobInfo => build_job_info_for_batch_job_data(bjd),
        :skipped => bjd[:skipped]
      }
    end
  end

  def build_job_info_for_batch_job_data(batch_job_data)
    job = batch_job_data.job
    return nil if job.nil?
    {
      :jobNumber => job.number,
      :side => job.side
    }
  end

  # Batch save methods

  def self.build_batch_runs_for_runs(runs)
    runs.map do |run_data|
      {
        run_key: run_data[:run_key]
      }
    end
  end

  def self.build_batch_containers_for_sector_data(sector_data)
    sector_data.map do |sector_datum|
      {
        external_container_id: sector_datum[:containerId],
        order: sector_datum[:order]
      }
    end
  end

  def self.build_run_data_for_run(run)
    run.map do |rKey, rVal|
      next if rKey.to_s == 'run_key'
      {
        name: rKey,
        value: rVal.to_s
      }
    end.compact
  end

  def self.build_batch_container_data_for_sector_datum(sector_datum)
    sector_datum.map do |sKey, sVal|
      next if %w[containerId order lensData].include?(sKey.to_s)
      {
        name: sKey,
        value: sVal
      }
    end.compact
  end

  def self.build_batch_job_data_for_sector_datum(sector_datum)
    lens_data = sector_datum[:lensData]
    nil if lens_data.nil?
    lens_data.map do |lens_datum|
      job_info = lens_datum[:jobInfo]
      job_id = job_info.nil? ? nil : FindHelpers::JobFinders.find_active_or_create_by_number_and_side_num(job_info[:jobNumber], Sideable.side_letter_to_side_num(job_info[:side])).id
      {
        job_id: job_id,
        position: lens_datum[:position],
        skipped: lens_datum[:skipped]
      }
    end
  end
end
