# == Schema Information
#
# Table name: error_filters
#
#  id                 :bigint           not null, primary key
#  behaviour          :integer
#  enabled            :boolean          default(TRUE)
#  error_codes        :string(4000)
#  filter_target_type :string(4000)
#  group_code         :string(4000)
#  name               :string(4000)
#  replace_name       :string(4000)
#  system_default     :boolean          default(FALSE)
#  created_at         :datetime         not null
#  updated_at         :datetime         not null
#  filter_target_id   :bigint
#
# Indexes
#
#  index_error_filters_on_error_codes                              (error_codes)
#  index_error_filters_on_filter_target                            (filter_target_type,filter_target_id)
#  index_error_filters_on_filter_target_type_and_filter_target_id  (filter_target_type,filter_target_id)
#  index_error_filters_on_group_code                               (group_code) UNIQUE
#
class ErrorFilter < ApplicationRecord
  belongs_to :filter_target, polymorphic: true

  enum behaviour: [:ignore, :replace, :use_as_is]

  validates :group_code, uniqueness: true

  def should_broadcast?
    true
  end

  def is_ignore_filter?
    behaviour == 'ignore'
  end

  def self.calculate_filter_for_error(error)
    machine = error.machine

    filter = nil

    # Prioritize machine group filters over machine type filters
    filter = find_filter_for_target(error, :MachineGroup, machine.machine_group_ids) if machine.belongs_to_group?
    return filter unless filter.nil?

    filter = find_filter_for_target(error, :MachineType, machine.machine_type_id) if machine.has_machine_type?

    filter
  end

  private

  def self.find_filter_for_target(error, target_type, target_ids)
    # Try to find an exact match
    filter = ErrorFilter.where(filter_target_type: target_type, filter_target_id: target_ids, enabled: true)
                        .where(ErrorFilter.arel_table[:error_codes].matches_any(err_no_to_matches(error.errno))).limit(1).first

    return filter unless filter.nil?

    # Try to find a range match
    filter = ErrorFilter.where(filter_target_type: target_type, filter_target_id: target_ids, enabled: true)
               .where(ErrorFilter.arel_table[:error_codes].matches("%-%")).find_each
               .find do |error_filter|

      # Extract the ranges from the rule
      ranges = error_filter.error_codes.split(/(?:,\s*|,)/).filter {|err| err.include? "-"}

      #Try to find a range that matches our errorno
      ranges.find do |range|
        bounds = range.split("-").map { |e| e.to_i }
        (bounds[0]..bounds[1]).include?(error.errno)
      end
    end
    filter
  end

  # MSSQL does not support regex, so we have to find exact patterns.
  def self.err_no_to_matches(err_no)
    %W[#{err_no} %,#{err_no} %,#{err_no},% #{err_no},%]
  end
end
