module ReportComponents
  class DrlTable < ReportComponent
    def component_id
      'drl_table'
    end

    def component_type
      'table'
    end

    def generate(args)

      events_grouped_by_machine = MachineKpi.event_drl_finished_between(start_time, end_time, timeframe, machine_ids, options: args)

      rows = []

      if events_grouped_by_machine.blank?
        return set_no_data('No DRL found during period.')
      end


      events_grouped_by_machine.each do |machine_sn, events|
        current_day_root_row = nil
        current_row_day = nil

        current_day_passed = 0
        current_day_failed = 0

        events.each do |event|
          event_datetime = Time.zone.parse(event[:time])

          row = build_event_row(machine_sn, event)

          if current_day_root_row.nil?
            # Keep a reference to the first row of a day to later set the first pass yield
            current_day_root_row = row
            current_row_day = event_datetime.to_date
          end

          rows.push(row)

          # We're changing row, tally up the first pass yield and update the current day row
          if current_row_day != event_datetime.to_date
            current_day_root_row[:first_yield][:value] = first_pass_yield(current_day_passed, current_day_failed)

            current_day_passed = 0
            current_day_failed = 0

            current_day_root_row = row
            current_row_day = event_datetime.to_date
          end

          if event[:result].downcase == 'passed'
            current_day_passed += 1
          elsif event[:result].downcase == 'failed'
            current_day_failed += 1
          end
        end

        # Tally up the last day for this machine
        current_day_root_row[:first_yield][:value] = first_pass_yield(current_day_passed, current_day_failed)
      end

      {
        series: rows
      }
    end

    def build_event_row(machine_sn, event)
      prod_settings = ProductionSetting.instance

      time = Time.zone.parse(event[:time])
      month = time&.strftime("%B")
      day = time&.strftime("%d")

      {
        blocker_sn: {
          value: machine_sn
        },
        result: {
          value: event[:result],
          color: colorize_bool(event[:result]&.downcase == 'passed')
        },
        tx: {
          value: event[:tx],
          color: colorize_num(event[:tx], prod_settings.tx_tolerance)
        },
        ty: {
          value: event[:ty],
          color: colorize_num(event[:ty], prod_settings.ty_tolerance)
        },
        rz: {
          value: event[:rz],
          color: colorize_num(event[:rz], prod_settings.rz_tolerance)
        },
        month: {
          value: month
        },
        day: {
          value: day
        },
        time: {
          value: time&.strftime("%H:%M:%S")
        },
        first_yield: {
          value: nil
        }
      }
    end

    def first_pass_yield(successes, failures)
      return 0.0 if successes == 0
      return 100.0 if successes == 1 && failures == 0;

      Tools.percentage_maker(1.0, failures + 1, 2)
    end

    def success_color
      '#80FF80'
    end

    def failed_color
      '#FF8080'
    end

    def colorize_bool(is_valid)
      return success_color if is_valid
      failed_color
    end

    def colorize_num(value, target)
      return colorize_bool(value.to_f.abs < target)
    end
  end
end