module ReportComponents
  class MachinePerformanceBatch < ReportComponent
    include DataHelpers
    include ReportComponentHelpers
    include KpiHelpers

    class InvalidTimeFrame < StandardError; end

    def component_id
      'machine_performance_batch'
    end

    def component_type
      if use_timeline?
        'range-bar'
      else
        'bar'
      end
    end

    def generate(args)
      if use_timeline?
        build_timeline_component
      else
        build_count_component
      end
    end

    private

    def build_count_component
      @component_data = MachineKpi.box_coater_batches_count_between(start_time, end_time, timeframe, machine_ids, options: {aggregate_machines: true})

      categories = []
      finished = []
      aborted = []
      overall_count = 0
      overall_duration = 0

      @component_data.each do |period, batch_info|
        categories << bucket_time_to_timeframe_label(period)
        finished << batch_info[:finished]
        aborted << batch_info[:aborted]
        overall_count += (batch_info[:finished] + batch_info[:aborted])
        overall_duration += batch_info[:durationOverPeriod]
      end

      batch_duration_avg = if overall_count == 0
                             0
                           else
                             # Each day is already an average, so do the average of averages
                             ((overall_duration / 1.minutes) / overall_count).floor
                           end

      {
        options: {
          graphVariant: 'batch-count'
        },
        subtitle: build_subtitle_legend(overall_count, batch_duration_avg),
        categories: categories,
        series: [
          {
            name: 'Finished',
            data: finished
          },
          {
            name: 'Aborted',
            data: aborted
          }
        ]
      }
    end

    def build_timeline_component
      @component_data = MachineKpi.box_coater_batches_run_between(start_time, end_time, machine_ids, options: {aggregate_machines: false})

      if machine_ids.size == 1
        build_single_machine_timeline
      else
        build_multi_machine_timeline
      end
    end

    def use_timeline?
      timeframe == Timeframe::HOUR_BY_HOUR
    end

    def build_single_machine_timeline
      machine_id = machine_ids[0]

      duration_total = 0

      batches_info = @component_data[machine_id].each_with_index.map do |batch_data, i|
        duration = batch_duration(batch_data[:start], batch_data[:end])
        duration_total += duration

        {
          name: "Batch #{i+1}",
          state: batch_data[:state],
          x: "Batch #{i+1}",
          y: [
            batch_data[:start].to_s,
            batch_data[:end].to_s
          ],
          fillColor: fill_color_for_state(batch_data[:state]),
          duration: duration
        }
      end

      batch_avg_duration = if batches_info.size == 0
                             0
                           else
                             (duration_total / batches_info.size).floor
                           end

      {
        options: {
          graphVariant: 'single-machine-timeline'
        },
        subtitle: build_subtitle_legend(batches_info.size, batch_avg_duration),
        series: [
          {
            name: Machine.find(machine_id).extended_name,
            data: if batches_info.size == 0
                    [
                      {
                        x: '',
                        y: [],
                        duration: -1
                      }
                    ]
                  else
                    batches_info
                  end
          }
        ]
      }
    end

    def build_multi_machine_timeline
      all_batches = []
      duration_total = 0
      empty_batches = 0

      @component_data.each do |machine_id, batches|
        machine_name = Machine.find(machine_id).extended_name

        if batches.size > 0
          batches.each_with_index do |batch_data, i|
            duration = batch_duration(batch_data[:start], batch_data[:end])
            duration_total += duration

            all_batches << {
              name: "#{machine_name} - Batch #{i+1}",
              state: batch_data[:state],
              x: machine_name,
              y: [
                batch_data[:start].to_s,
                batch_data[:end].to_s
              ],
              fillColor: fill_color_for_state(batch_data[:state]),
              duration: duration
            }
          end
        else
          empty_batches += 1 # do not count empty batches for the average
          all_batches << {
            x: machine_name,
            y: [],
            duration: -1
          }
        end
      end

      batch_average_duration = if all_batches.size == 0
                                 0
                               else
                                 (duration_total / (all_batches.size - empty_batches)).floor
                               end

      {
        options: {
          graphVariant: 'multi-machine-timeline'
        },
        subtitle: build_subtitle_legend(all_batches.size, batch_average_duration),
        series: [
          {
            name: 'Batches',
            data: all_batches
          },
        ]
      }
    end

    def group_for_state(state)
      return 'Finished' if state == 'finished' || state == 'ongoing'
      return 'Aborted' if state == 'aborted'
      'unknown'
    end

    def fill_color_for_state(state)
      return '#1a5dad' if state == 'finished'
      return '#ff0000' if state == 'aborted'
      return '#888888' if state == 'ongoing'
      '#000'
    end

    def batch_duration(start_time, end_time)
      ((end_time - start_time) / 1.minutes).floor
    end

    def build_subtitle_legend(batch_count, batch_avg_duration)
      return "Total Batch Runs: 0" if batch_count == 0

      "Total Batch Runs: #{batch_count} (avg. duration: #{batch_avg_duration} min.)";
    end
  end
end

