Rails.application.routes.draw do
  mount Rswag::Ui::Engine => '/api-docs'
  mount Rswag::Api::Engine => '/api-docs'
  mount RxModule::Engine => "/api/rx_module"

  scope :api, defaults: {format: :json} do
    resources :machine_groups
    resources :machine_types, only: [:index, :show]
    resources :colors
    resources :job_breakages, only: [:create, :index, :show] do
      collection do
        get :search
      end
      member do
        put :validate
      end
    end
    resources :blocks, only: [:index, :show] do
      member do
        put "reactivate"
        put "discard"
        put "wash"
        put "force_to_clean"
      end
      collection do
        get "find_by_dmc"
        post "import"
      end
    end
    resources :block_pickings do
      member do
        post 'search' => 'block_pickings#search'
      end
    end
    resources :process_booking_workstations do
      member do
        post :book
      end
    end

    resources :block_picking_tags
    resources :shifts, only: [:index, :show]
    resources :machines do
      collection do
        get "status_all" => "machines#status_all"
      end
      member do
        put "archive" => "machines#archive"
        put 'restore' => "machines#restore"
        get :machine_last_state

        # EER routes
        put "register_as_eer_machine"
        get "box_coater_event"

        put "engineering_lockout" => "machines#engineering_lockout"
        get "engineering_lockout_status" => "machines#engineering_lockout_status"

        # Machine Prod Data Reports
        get :production_data
      end
    end

    resources :box_coater_events, only: [:show, :update, :index] do
      collection do
        put :latest_for_machines
      end
    end

    resources :box_coater_recipes

    resources :eer_yields do
      collection do
        post "yield_exists"
      end
    end
    resources :calibrations
    resources :events do
      member do
        get :event_data
      end
    end
    resources :maintenances
    resources :errors
    resources :machine_states
    resources :job_states do
      member do
        get :process_params
      end
      collection do
        put :add_bulk
      end
    end
    resources :jobs do
      member do
        get "latest_routing_states"
      end
      get "states"
      get "process_data"
      patch "/revive" => "jobs#revive"
      collection do
        get "job_history"
        get "jobs_for_tray_number"
        get :routing_launch_data
      end
    end
    resources :job_process_parameters
    resources :job_processes, only: [:index, :show]
    resources :order_data
    resources :addresses
    resources :orders do
      collection do
        post :create_from_routing
        post :bulk_order_data
      end
      patch "/revive" => "orders#revive"
    end
    resources :job_data
    resources :capability_groups
    resources :capabilities
    resources :manufacturing_processes
    resources :production_settings
    resources :data_archive_settings
    resources :block_reuse_settings do
      collection do
        put "start_server"
        put "stop_server"
      end
    end
    resources :push_targets
    scope "/template_report" do
      post "generate_template_reports", to: "template_report#generate_template_reports"
      post "single_report_generate", to: "template_report#single_report_generate"
    end

    post "register_push_targets" => "push_targets#register_push_targets"

    resources :machine_machine_data_push_targets, :only => [:index, :destroy]
    resources :machine_data_push_targets do
      member do
        put "add_machines"
      end
    end
    resources :error_filters do
      member do
        put "toggle"
      end
    end
    resources :abort_filters do
      member do
        put "toggle"
      end
    end

    resources :batches, only: [:create, :index, :show] do
      collection do
        get :batch_data_as_json
        get :search_by_job_number
      end
    end

    # Rx-Net uses two urls to send the job, either this one or the /lms/2.0, so send both to add_order_and_job. We will want to retire this one eventually.
    scope "/connectors/rxnet" do
      put "orders", to: "orders#add_order_and_jobs"
    end

    post "register_push_targets" => "push_targets#register_push_targets"
    get "/jobs_status" => "jobs#jobs_status"
    patch "register_as_eer_lab" => "production_settings#register_as_eer_lab"

    scope "/export/machines/:machine_id" do
      post "/jobs" => "exports#get_jobs"
      post "/machine_states" => "exports#get_machine_states"
      post "/job_states" => "exports#get_job_states"
      post "/errors" => "exports#get_errors"
      post "/events" => "exports#get_events"
      post "/maintenances" => "exports#get_maintenances"
      post "/calibrations" => "exports#get_calibrations"
      post "/process_data" => "exports#get_process_data"
    end
  end

  resources :calibration_corrections
  resources :calibration_measurements
  resources :calibrations

  namespace "data_export_api", path: 'export' do
    # External api endpoints
    namespace "v1", path: '1.0' do
      #Gets all the actions added in the external_api controller and makes a route for them
      DataExportApi::V1::ExternalApiController.action_methods.difference(ApiController.action_methods).each do |action|
        match "#{action}" => "external_api##{action}", via: [:get, :post]
      end
    end
  end

  scope "/connector/satisloh", defaults: {format: :json} do
    # Compatibility endpoints for MES API
    put "machinestate/2.0/add", to: "machine_states#add", as: :add_machine_state
    put "jobstate/1.0/add", to: "job_states#add", as: :job_state_add
    put "error/1.0/add", to: "errors#add", as: :add_error
    put "maintenance/1.0/add", to: "maintenances#add", as: :add_maintenance
    put "event/1.0/add", to: "events#add", as: :add_event
    put "processparameter/1.1/add", to: "job_process_parameters#add", as: :add_process_parameters
    post "coating/1.0/batch/generate", to: "coating#generate", as: :generate_batch
    post "capability/1.0/:sn", to: "capabilities#machine_update", as: :machine_update, sn: /[^\/]+/


    # The S4P api has two endpoints to add calibration that do the same thing, so we have to support both
    put "calibration/1.0/add", to: "calibrations#add", as: :add_calibration
    post "calibration/1.0/:sn", to: "calibrations#add"

    put "/lms/1.0/order/:id", to: "orders#add", as: :add_order
    put "/lms/1.0/job/:job_id/:lens_id", to: "jobs#add_to_order", as: :add_lens_to_job
    put "/lms/1.0/job/:id", to: "jobs#add", as: :add_job
    put "/lms/2.0/order", to: "orders#add_order_and_jobs", as: :add_order_and_jobs

    # Note: the old S4P system had two endpoint for this for some reason, so keep them for legacy but point to the same action
    put "/jobinfo/1.0/blockinfo", to: "blocks#add"
    put "/jobinfo/1.0/component/blockinfo", to: "blocks#add", as: :add_block



    # TODO: deprecate this
    get "/reportqueries/resources/groups/1.0/list", to: "machine_groups#list", as: :get_machine_groups_list
    get "/reportqueries/resources/status/1.0/last", to: "machines#status_last", as: :get_machines_status_last
    get "/reportqueries/resource/:sn/1.0/processinfo", to: "machines#process_info", as: :get_machine_process_info, sn: /[^\/]+/
    get "/reportqueries/resource/:sn/1.0/jobstates", to: "machines#job_states", as: :get_machine_job_states, sn: /[^\/]+/
    get "/reportqueries/resource/:sn/1.0/jobdata", to: "machines#job_data", as: :get_machine_job_data, sn: /[^\/]+/
    get "/reportqueries/resource/:sn/1.0/jobprocessparameters", to: "machines#job_process_parameters", as: :get_machine_job_process_parameters, sn: /[^\/]+/
    get "/reportqueries/resource/:sn/1.0/jobs", to: "machines#jobs", as: :get_machine_jobs, sn: /[^\/]+/

    # Adding this endpoint as a fix for lab overview when calculating for process info
    # This endpoint is being called in the calculation but can't reach anything so it throws
    # Currently set to just return an empty array since we don't record job_breakages in rx-data as of writing this.
    # Might be worth looking into again in the future should we start to track job_breakages in rx-data
    get "/reportqueries/jobs/breakage/1.0/list", to: "jobs#breakages", as: :get_breakages

    get "/jobinfo/1.0/order/jobs", to: "orders#jobs", as: :order_jobs

    get "/jobinfo/1.0/order/data", to: "orders#data", as: :mes_order_data
    get "/jobinfo/1.0/ordertype/list", to: "orders#types", as: :order_types

    get "/resources/1.0/status/colors", to: "colors#colors", as: :mes_colors

    get "/error/1.0", to: "errors#errors", as: :mes_errors
    get "/event/1.0/list", to: "events#mes_events", as: :mes_events
    # TODO: /reportqueries/routing/history/2.0/jobs'
    get "/reportqueries/resources/status/1.0/history", to: "machines#status_history", as: :mes_status_history
    get "/reportqueries/routing/history/2.0/jobs", to: "jobs#routing_history", as: :mes_routing_history

    # Need to add time endpoint as it is in use for MES_HW_COMM_BOX and some other machines
    get "/time/1.0", to: "application#time"

    # The following endpoints are deprecated and are only in the system to support legacy
    delete "/lms/1.0/order/:id", to: "orders#delete", as: :delete_order
    delete "/lms/1.0/job/:id", to: "jobs#delete", as: :delete_job
    delete "/lms/1.0/job/:job_id/:lens_id", to: "jobs#delete_from_order", as: :delete_job_from_order
    # endpoint disabled as of 9/11/2020. See controller action for notes
    # get '/jobinfo/1.0/component/processlist', to: "jobs#processes", as: :mes_job_processes

    # End compatibility endpoints
  end

  root "/", controller: :index, action: :show, as: :index_show
  match "*path" => "index#show", :via => :all
end
