# frozen_string_literal: true

require 'rails_helper'

RSpec.configure do |config|
  # Specify a root folder where Swagger JSON files are generated
  # NOTE: If you're using the rswag-api to serve API descriptions, you'll need
  # to ensure that it's configured to serve Swagger from the same folder
  config.swagger_root = Rails.root.join('swagger').to_s

  # Define one or more Swagger documents and provide global metadata for each one
  # When you run the 'rswag:specs:swaggerize' rake task, the complete Swagger will
  # be generated at the provided relative path under swagger_root
  # By default, the operations defined in spec files are added to the first
  # document below. You can override this behavior by adding a swagger_doc tag to the
  # the root example_group in your specs, e.g. describe '...', swagger_doc: 'v2/swagger.json'
  config.swagger_docs = {
    'v1/swagger.yaml' => {
      openapi: '3.0.1',
      info: {
        title: 'Rx-Data API v1',
        version: 'v1',
        description: 'It is recommended to use the post variation of each endpoint because of the lack of limitation on the body size that exists on get requests in the query size (2000 characters)'
      },
      servers: [
        {
          url: '/export/1.0',
        }
      ],
      components: {
        schemas: {
          machine_state_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              sn: { '$ref' => '#/components/schemas/sns' },
              state: { '$ref' => '#/components/schemas/machine_states' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            },
            required: %w[start end sn]
          },
          machine_state_resp: {
            type: :object,
            properties: {
              machine_states: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    sn: { '$ref' => '#/components/schemas/sn' },
                    state: { '$ref' => '#/components/schemas/machine_state' },
                    time: { '$ref' => '#/components/schemas/time' },
                    calculated_duration: { '$ref' => '#/components/schemas/calculated_duration' },
                    comment: { '$ref' => '#/components/schemas/comment' },
                    addon: { '$ref' => '#/components/schemas/addon' }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },
          job_state_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              sn: { '$ref' => '#/components/schemas/sns' },
              job: { '$ref' => '#/components/schemas/jobs' },
              state: { '$ref' => '#/components/schemas/job_states' },
              side: { '$ref' => '#/components/schemas/side' },
              process: { '$ref' => '#/components/schemas/processes' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            },
            required: ["start", "end"]
          },
          job_state_resp: {
            type: :object,
            properties: {
              job_states: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    job: { '$ref' => '#/components/schemas/job' },
                    state: { '$ref' => '#/components/schemas/job_state' },
                    substate: { '$ref' => '#/components/schemas/substate' },
                    process: { '$ref' => '#/components/schemas/process' },
                    comment: { '$ref' => '#/components/schemas/comment' },
                    scrap: { '$ref' => '#/components/schemas/scrap' },
                    scrap_reason: { '$ref' => '#/components/schemas/scrap_reason' },
                    time: { '$ref' => '#/components/schemas/time' },
                    sn: { '$ref' => '#/components/schemas/sn' }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },
          process_params_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              sn: { '$ref' => '#/components/schemas/sns' },
              job: { '$ref' => '#/components/schemas/jobs' },
              param: { '$ref' => '#/components/schemas/param' },
              side: { '$ref' => '#/components/schemas/side' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            },
            required: %w[start end]
          },
          process_params_resp: {
            type: :object,
            properties: {
              process_params: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    job: { '$ref' => '#/components/schemas/job' },
                    side: { '$ref' => '#/components/schemas/side' },
                    sn: { '$ref' => '#/components/schemas/sn' },
                    time: { '$ref' => '#/components/schemas/time' },
                    params: {
                      type: :array,
                      items: { '$ref' => '#/components/schemas/process_param' }
                    }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },
          machine_errors_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              sn: { '$ref' => '#/components/schemas/sns' },
              job: { '$ref' => '#/components/schemas/jobs' },
              lensside: { '$ref' => '#/components/schemas/side' },
              errno: { '$ref' => '#/components/schemas/errnos' },
              errortype: { '$ref' => '#/components/schemas/errortypes' },
              status: { '$ref' => '#/components/schemas/statuses' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            },
            required: %w[start end sn]
          },
          machine_errors_resp: {
            type: :object,
            properties: {
              machine_errors: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    sn: { '$ref' => '#/components/schemas/sn' },
                    time: { '$ref' => '#/components/schemas/time' },
                    job: { '$ref' => '#/components/schemas/job' },
                    side: { '$ref' => '#/components/schemas/side' },
                    error: { '$ref' => '#/components/schemas/error' },
                    errno: { '$ref' => '#/components/schemas/errno' },
                    errortype: { '$ref' => '#/components/schemas/errortype' },
                    status: { '$ref' => '#/components/schemas/status' },
                    piececounter: { '$ref' => '#/components/schemas/piececounter' }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },
          lens_attribute_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              job: { '$ref' => '#/components/schemas/jobs' },
              side: { '$ref' => '#/components/schemas/side' },
              attribute: { '$ref' => '#/components/schemas/attribute_names' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            }
          },
          lens_attribute_resp: {
            type: :object,
            properties: {
              lens: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    job: { '$ref' => '#/components/schemas/job' },
                    side: { '$ref' => '#/components/schemas/side' },
                    time: { '$ref' => '#/components/schemas/time' },
                    attributes: { '$ref' => '#/components/schemas/attributes' }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },
          block_pieces_body: {
            type: :object,
            properties: {
              start: { '$ref' => '#/components/schemas/start' },
              end: { '$ref' => '#/components/schemas/end' },
              limit: { '$ref' => '#/components/schemas/limit' },
              offset: { '$ref' => '#/components/schemas/offset' },
              order: { '$ref' => '#/components/schemas/order' }
            },
            required: %w[start end]
          },
          block_pieces_resp: {
            type: :object,
            properties: {
              blocks: {
                type: :array,
                items: {
                  type: :object,
                  properties: {
                    dmc: { '$ref' => '#/components/schemas/dmc' },
                    usecount: { '$ref' => '#/components/schemas/usecount' },
                    time: { '$ref' => '#/components/schemas/time' }
                  }
                }
              },
              next_offset: { '$ref' => '#/components/schemas/next_offset' }
            }
          },


          ### Base types
          sn: {
            type: :string,
            example: '27871'
          },
          sns: {
            type: :array,
            items: { '$ref' => '#/components/schemas/sn' },
            example: %w[27871 BLP001]
          },
          job: {
            type: :string,
            example: '153846'
          },
          jobs: {
            type: :array,
            items: { '$ref' => '#/components/schemas/job' },
            example: %w[153846 51369]
          },
          time: {
            type: :string,
            format: :date,
            example: '2022-03-01T00:00:00,000'
          },
          calculated_duration: {
            type: :number,
            example: 600.00
          },
          start: {
            type: :string,
            format: :date,
            example: '2022-03-01'
          },
          end: {
            type: :string,
            format: :date,
            example: '2022-03-04T23:59:59,000'
          },
          job_state: {
            type: :string,
            example: 'finished',
            enum: %w[started aborted finished postponed skipped restarted]
          },
          job_states: {
            type: :array,
            items: { '$ref' => '#/components/schemas/job_state' },
            example: %w[started finished]
          },
          machine_state: {
            type: :string,
            example: 'waitingforjobs',
            enum: %w[on notready ready waitingforjobs processing maintenance error halt estop setup off]
          },
          machine_states: {
            type: :array,
            items: { '$ref' => '#/components/schemas/machine_state' },
            example: %w[notready waitingforjobs]
          },
          state: {
            type: :string,
            example: 'finished'
          },
          substate: {
            type: :string,
            example: 'washed'
          },
          side: {
            type: :string,
            enums: ['left', 'right'],
            example: 'left'
          },
          processes: {
            type: :array,
            items: { '$ref' => '#/components/schemas/process' },
            example: %w[blocking generating]
          },
          process: {
            type: :string,
            example: 'generating'
          },
          comment: {
            type: :string,
            example: ''
          },
          addon: {
            type: :string,
            example: ''
          },
          params: {
            type: :array,
            items: { '$ref' => '#/components/schemas/param' },
            example: %w[MacroName HeatingTime]
          },
          param: {
            type: :string,
            example: "MacroName"
          },
          scrap: {
            type: :boolean,
            example: false
          },
          scrap_reason: {
            type: :string,
            example: ''
          },
          error: {
            type: :string,
            example: 'Ran out of cooling'
          },
          errno: {
            type: :number,
            example: '153845'
          },
          errnos: {
            type: :array,
            items: { '$ref' => '#/components/schemas/errno' },
            example: %w[153845 153846]
          },
          errortype: {
            type: :string,
            example: 'alarm',
            enums: %w[unknown info alarm error]
          },
          errortypes: {
            type: :array,
            items: { '$ref' => '#/components/schemas/errortype' },
            example: %w[info alarm]
          },
          status: {
            type: :string,
            example: 'raised',
            enums: %w[raised acknowledged finished]
          },
          statuses: {
            type: :array,
            items: { '$ref' => '#/components/schemas/status' },
            example: %w[acknowledged finished]
          },
          piececounter: {
            type: :number,
            example: 1
          },
          attribute_names: {
            type: :array,
            items: { '$ref' => '#/components/schemas/attribute' },
            example: %w[LIND xValue]
          },
          attribute: {
            type: :string,
            example: "LIND"
          },
          value: {
            type: :string,
            example: "1.0"
          },
          attributes: {
            type: :array,
            items: {
              type: :object,
              properties: {
                name: { '$ref' => '#/components/schemas/attribute' },
                value: { '$ref' => '#/components/schemas/value' }
              }
            }
          },
          dmc: {
            type: :string,
            example: "11112542154541214"
          },
          usecount: {
            type: :number,
            example: 49
          },
          process_param: {
            type: :object,
            properties: {
              value: {
                type: :string,
                example: '1.0'
              },
              name: {
                type: :string,
                example: 'LIND'
              }
            }
          },
          limit: {
            type: :integer,
            default: 1000
          },
          offset: {
            type: :integer,
            default: 0
          },
          order: {
            type: :string,
            default: 'DESC',
            enums: ['DESC', 'ASC']
          },
          next_offset: {
            type: :integer,
            example: -1
          }
        }
      }
    }
  }

  # Specify the format of the output Swagger file when running 'rswag:specs:swaggerize'.
  # The swagger_docs configuration option has the filename including format in
  # the key, this may want to be changed to avoid putting yaml in json files.
  # Defaults to json. Accepts ':json' and ':yaml'.
  config.swagger_format = :yaml
end
